#include <rgb_lcd.h>
#include <Keypad.h>

/*
JUEGO SIMON

En cada intento, se genera un nuevo número entre 1 y 4 que se añade a la secuencia
que el jugador debe seguir o completar
La secuencia la almacenamos en un arreglo.
La secuencia máxima es de 30 elementos

Se van a usar 4 leds, el keypad, la pantalla LCD.
El keypad va conectado entre en los pines 13 a 6 (o a 5) por ejemplo:
13, 12, 11, 10, 8, 7, 6, 5

Los leds vamos a conectar a los pines 4,  2, 15 16. Los pines 15 y 16 ocupan las
posciones de A1 y A2.

A0 lo dejamos libre para usarlo como semilla del número random a generar

El buzzer lo vamos a conectar al pin 3 (D3)

Estados del juego

Estado 0:
Pantalla de bienvenida. Inicio del juego.

Estado 1:
Generación de un elemento más de la secuencia. Encendido de los leds correspondientes

Estado 2:
El jugador trata de imitar la secuencia presentado ajustado las teclas respectivas
Si se equivoca, pasamos al estado 3, que es cuando el usuario termina el juego porque
no adivinó la secuencia

Si no se equivoca, regresamos a estado 1.

Estado 3;
El jugador se equivocó, detenemos el juego e indicamos cuántos intentos logró hacer.  Reiniciamos el juego

*/

/******* KEYPAD *********/ 
const byte ROWS = 4; //four rows
const byte COLS = 4; //four columns

//matriz de mapeo del teclado
char keys[ROWS][COLS] = {
  {'1','2','3','4'},
  {'A','5','6','B'},
  {'7','8','9','C'},
  {'*','0','#','D'}
};

//conexión de filas y columnas del keypad al Arduino
byte rowPins[ROWS] = {13, 12, 11, 10}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {8, 7, 6, 5}; //connect to the column pinouts of the keypad

// creación del objeto teclado
Keypad teclado = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );

/****** PANTALLA LCD *******/
rgb_lcd pantalla;

/****** VARIABLES PARA EL JUEGO *******/
int secuencia[30];  //arreglo para la secuencia generada
int secuenciaJugador[30];  //secuencia ingresada por el jugador
int intentos;
int estado;
int tiempoSecuencia; //tiempo en que se muestra la secuencia;
int tiempoUsuario; //tiempo que tiene el usuario para ingresar la sencuencia
int tiempoEncendido; //tiempo que está encendido cada led
long tiempo;  //contabiliza el tiempo transcurrido
int numerosIngresados; //cantidad de números que debe ingresar el usuario para completar la secuencia en un cierto momento

/***** LEDS y BUZZER ******/
int buzzer = 3;
int leds[5] = {0, 2, 4, 15, 16}; //pines a los que están conectados los leds (el primer no cuenta)

void inicializar(){
  pantalla.clear();
  pantalla.setRGB(0, 0, 0);
  estado = 0;
  intentos = 0;
  tiempoSecuencia = 1000;
  tiempoUsuario = 3000;
  tiempoEncendido = 800;
  numerosIngresados = 0;
}

void setup() {
  pantalla.begin(16, 2);
  pinMode(buzzer, OUTPUT);
  for (int i=1; i<5; i++){
    pinMode(leds[i], OUTPUT);
  }

  inicializar();

}

void loop() {
  char tecla = teclado.getKey();

  if (estado == 0){
    pantalla.setCursor(0, 0);
    pantalla.print("SIMON");
    pantalla.setCursor(0, 1);
    pantalla.print("Press B to begin");

    if (tecla && tecla == 'B'){
      //generación de un número de la secuencia
      randomSeed(millis()/100/analogRead(0));
      secuencia[intentos] = random(1, 5);
      pantalla.clear();
      estado = 1;
    }



  }else if (estado == 1){
    pantalla.setCursor(0, 0);
    pantalla.print("secuencia");
    pantalla.setCursor(0, 1);
    pantalla.print("intentos: ");
    pantalla.print(intentos);

    //encendido de los leds de la secuencia
    for (int i=0; i<=intentos; i++){
     digitalWrite(leds[secuencia[i]], HIGH);
     delay(tiempoSecuencia);
     digitalWrite(leds[secuencia[i]], LOW);
     delay(tiempoSecuencia);
   }

   pantalla.clear();
   numerosIngresados = 0;
   estado = 2;

  }else if (estado == 2){
    pantalla.setCursor(0, 0);
    pantalla.print("usuario");
    pantalla.setCursor(0, 1);
    pantalla.print("intentos: ");
    pantalla.print(intentos);
    String numeros = "";

    if (tecla && (tecla == '1' || tecla == '2' || tecla == '3' || tecla == 'A') && numerosIngresados <= intentos){
      numeros += tecla;
      secuenciaJugador[intentos] = numeros.toInt();
      digitalWrite(leds[secuenciaJugador[numerosIngresados]], HIGH);
      delay(tiempoSecuencia);
      digitalWrite(leds[secuenciaJugador[numerosIngresados]], LOW);
      
      //comparamos el número ingresado con el número correspondiente de la secuencia
      if (secuenciaJugador[numerosIngresados] != secuencia[numerosIngresados]){
        //hay un error. El jugador perdió vamos a estado 2
        pantalla.clear();
        estado = 3;
      }

      numerosIngresados++;
      if (numerosIngresados > intentos){
        //regresamos a estado 1 para generar un nuevo número y volver a adivinar
        intentos++;
        numerosIngresados = 0;
        randomSeed(millis()/100/analogRead(0));
        secuencia[intentos] = random(1, 5);
        pantalla.clear();
        estado = 1;

      }
  
    }



  }else if (estado == 3){


  }

  delay(200);


}
